package com.jz.agent.client.service;

import com.google.gson.Gson;
import com.jz.agent.client.DmpDsAgentClient;
import com.jz.agent.utils.JDBCUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.agent.ResultCode;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.security.UserGroupInformation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class DmpHiveAgentClientService implements DmpDsAgentClient {

    private Gson gson = new Gson();
    private static Logger logger = LoggerFactory.getLogger(DmpHiveAgentClientService.class);

    public DmpAgentResult testConnect(DmpAgentDatasourceInfo ds) {
        Connection connection = null;
        String jdbcUrl = ds.getJdbcUrl();
        try {
            jdbcUrl += securityConnect(ds);

            Class.forName(ds.getDriverClassName());
            connection = DriverManager.getConnection(jdbcUrl);
            connection.close();
            return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(true));
        } catch (Exception e) {
            logger.error("HIVE连通性异常", e);
            return new DmpAgentResult(ResultCode.EXCEPTION, "HIVE连通性异常：" + e.getMessage());
        } finally {
            JDBCUtils.disconnect(connection, null, null);
        }
    }

    public DmpAgentResult getTableNameList(DmpAgentDatasourceInfo ds) {
        List<String> list = new ArrayList();
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        String jdbcUrl = ds.getJdbcUrl();
        String sql = "show tables";
        try {
            jdbcUrl += securityConnect(ds);

            Class.forName(ds.getDriverClassName());
            connection = DriverManager.getConnection(jdbcUrl);
            ps = connection.prepareStatement(sql);
            rs = ps.executeQuery();
            while (rs.next()) {
                list.add(rs.getString("tab_name"));
            }
        } catch (Exception e) {
            logger.error("HIVE获取表列表异常", e);
            return new DmpAgentResult(ResultCode.EXCEPTION, "HIVE获取表列表异常：" + e.getMessage());
        } finally {
            JDBCUtils.disconnect(connection, rs, ps);
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
    }

    public DmpAgentResult getTableColumnList(DmpAgentDatasourceInfo ds, String tableName) {
        List<Map<String, Object>> list = new ArrayList();
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        String jdbcUrl = ds.getJdbcUrl();
        String sql = "DESCRIBE " + tableName;

        try {
            jdbcUrl += securityConnect(ds);

            Class.forName(ds.getDriverClassName());
            connection = DriverManager.getConnection(jdbcUrl);
            ps = connection.prepareStatement(sql);
            rs = ps.executeQuery();
            while (rs.next()) {
                Map<String, Object> m = new HashMap<String, Object>();
                String col_name = rs.getString("col_name");
                String data_type =  rs.getString("data_type");
                if(!"".equals(col_name) && data_type != null) {
                	m.put("name", rs.getString("col_name").toLowerCase());
                    m.put("type", rs.getString("data_type").toUpperCase());
                    m.put("comment", rs.getString("comment"));
                    list.add(m);
                }
            }
        } catch (Exception e) {
            logger.error("HIVE获取表属性异常", e);
            return new DmpAgentResult(ResultCode.EXCEPTION, "HIVE获取表属性异常：" + e.getMessage());
        } finally {
            JDBCUtils.disconnect(connection, rs, ps);
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
    }

    public DmpAgentResult previewData(DmpAgentDatasourceInfo ds, String tableName) {
        Map<String, Object> map = new HashMap();
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(map));
    }

    /**
     * 安全连接并返回jdbc连接url的安全请求后缀
     *
     * @param ds
     * @return jdbc连接url的安全请求后缀
     */
    private String securityConnect(DmpAgentDatasourceInfo ds) throws IOException {
        String jdbcUrlSuffix = "";
        // 与kudu大体无异，不同于安全校验
        if ("true".equalsIgnoreCase(ds.getKerberosIsenable())) {
            String principal = ds.getAccessId(); 
            String hiveKeytabPath = ds.getAccessKey(); // D:/keytab/hive.keytab
            String hiveUser = ds.getEndpoint(); // hive@CRNETTEST.COM

            jdbcUrlSuffix = ";principal=" + principal;
            System.setProperty("java.security.krb5.conf", ds.getKerberosKrb5Conf());
            System.setProperty("java.security.auth.login.config", ds.getKerberosJaasConf());
            Configuration conf = new Configuration();
            conf.set("hadoop.security.authentication", "kerberos");
            UserGroupInformation.setConfiguration(conf);
            UserGroupInformation.loginUserFromKeytab(hiveUser, hiveKeytabPath);
        }
        return jdbcUrlSuffix;
    }

    public static void main(String[] args) {
        Connection connection = null;
        ResultSet rs = null;
        PreparedStatement ps = null;
        String JDBC_DRIVER = "org.apache.hive.jdbc.HiveDriver";
        String CONNECT_URL = "jdbc:hive2://10.0.53.177:10000/tmp;principal=hive/jtjzvdra116@CRNETTEST.COM";
        // String sql = "show tables";
        String sql = "DESCRIBE rzf_report_rzfyinliu_yp";
        try {
            System.setProperty("java.security.krb5.conf", "D:/keytab/krb5.conf");
            System.setProperty("java.security.auth.login.config", "D:/keytab/jaas.conf");

            Configuration conf = new Configuration();
            conf.set("hadoop.security.authentication", "Kerberos");
            UserGroupInformation.setConfiguration(conf);
            UserGroupInformation.loginUserFromKeytab("hive@CRNETTEST.COM", "D:/keytab/hive.keytab");

            Class.forName(JDBC_DRIVER);
            connection = DriverManager.getConnection(CONNECT_URL);
            ps = connection.prepareStatement(sql);
            rs = ps.executeQuery();
            while (rs.next()) {
                // System.out.println(rs.getString("tab_name"));
                System.out.print(rs.getString("col_name").toLowerCase() + " = ");
                System.out.print(rs.getString("data_type").toUpperCase() + " = ");
                System.out.println(rs.getString("comment") + " = ");
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.disconnect(connection, rs, ps);
        }
    }

}
