package com.jz.dmp.modules.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.PageInfoResponse;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.DataSourceListReq;
import com.jz.dmp.modules.controller.DataIntegration.bean.TaskListPageDto;
import com.jz.dmp.modules.dao.DmpSyncingDatasourceDao;
import com.jz.dmp.modules.model.DmpSyncingDatasource;
import com.jz.dmp.modules.service.DmpSyncingDatasourceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 数据源配置(DmpSyncingDatasource)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-24 14:09:59
 */
@Service("dmpSyncingDatasourceService")
@Transactional
public class DmpSyncingDatasourceServiceImpl implements DmpSyncingDatasourceService {

    private static Logger logger = LoggerFactory.getLogger(DmpSyncingDatasourceServiceImpl.class);

    @Autowired
    private DmpSyncingDatasourceDao dmpSyncingDatasourceDao;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public DmpSyncingDatasource queryById(Integer id) {
        return this.dmpSyncingDatasourceDao.queryById(id);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DmpSyncingDatasource> queryAllByLimit(int offset, int limit) {
        return this.dmpSyncingDatasourceDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dmpSyncingDatasource 实例对象
     * @return 实例对象
     */
    @Override
    public DmpSyncingDatasource insert(DmpSyncingDatasource dmpSyncingDatasource) {
        this.dmpSyncingDatasourceDao.insert(dmpSyncingDatasource);
        return dmpSyncingDatasource;
    }

    /**
     * 修改数据
     *
     * @param dmpSyncingDatasource 实例对象
     * @return 实例对象
     */
    @Override
    public DmpSyncingDatasource update(DmpSyncingDatasource dmpSyncingDatasource) {
        this.dmpSyncingDatasourceDao.update(dmpSyncingDatasource);
        return this.queryById(dmpSyncingDatasource.getId());
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Override
    public boolean deleteById(Integer id) {
        return this.dmpSyncingDatasourceDao.deleteById(id) > 0;
    }

    /**
     * 数据源列表查询
     *
     * @param req
     */
    @Override
    public PageInfoResponse<DataSourceListDto> queryDataSourceListPage(DataSourceListReq req, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DataSourceListDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        req.setDatasourceName(req.getDatasourceName().trim());
        List<DataSourceListDto> list = dmpSyncingDatasourceDao.queryDataSourceListPage(req);
        PageInfo<DataSourceListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 批量删除数据源
     *
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public JsonResult delDataSourceById(String datasourceId) throws Exception {
        String[] ids = datasourceId.split(",");
        dmpSyncingDatasourceDao.delDataSourceById(ids);
        return new JsonResult();
    }
}